function p = permutations(X, F, interactions, center, n_perm)

% Does permutation test on the factors of a crossed experimental design
%
% Input: 
% Data matrix X           rows: subject; columens: variables
% Design matrix F         rows: level per factor for each factor
% Interactions            array with per row interacting factors
% center                  1: center 2: auto scale
% n_perm                  number of permutations
% 

% Output:
% array with p-values for factors and interactions
%
% GZ, 23 January 2012. Matlab 7.11.0
% Version 1.0
% Version 1.1  24 January 2012: added permutation test for interaction
% Version 1.2  1 Feb 2012: fixed estimate of interaction significance DJV
% Version 1.21 19 July 2012: fixed some comments
% Version 1.2.2 19 July 2013: Fixed bug to calcualte multiple interactions
%
% Copyright: Gooitzen Zwanenburg
% This code is available under APACHE Licence 2.0
% http://www.apache.org/licenses/LICENSE-2.0.html

tic
    size_data           = size(X);                              % size of the data matrix 
    n_levels            = max(F);                               % number of levels / factor
    n_interactions      = size(interactions,1);                 % number of interactions
    n_factors           = size(F,2);                            % number of factors  
    factors             = 1 : n_factors;                        % factors to be evaluated
    X_level_means       = cell(n_factors,1);                    % level_means per factor  
    X_permuted          = cell(n_factors,1);                    % permuted data per factor
    X_interaction_means = cell(n_interactions);                 % cell_means X_int - X_a - X_b
    ssq                 = zeros(n_perm + 1,n_factors);          % within sum of squares data matrix
    ssq_interaction     = zeros(n_perm + 1,n_interactions);     % within sum of squares data matrix
    p_factor            = zeros(1, n_factors);                  % p-values factors
    p_interaction       = zeros(1, n_interactions);             % p-values interactions
  
    % center/standardize the data
    if center == 1
        X = (X - ones(size_data(1),1)*mean(X));      % Centre
    elseif center == 2
        X = (X - ones(size_data(1),1)*mean(X))./...  % Standardize
        (ones(size_data(1),1)*std(X));
    end
    
    % Make structure with unchanging 'variables'
    Fixed.F              = F;
    Fixed.interactions   = interactions; 
    Fixed.n_factors      = n_factors;
    Fixed.n_levels       = n_levels;
    Fixed.n_interactions = n_interactions;

% Collect level means for the factors indicated in the model 
    for factor = factors 
        X_level_means{factor} = level_means(X, Fixed, factor);
    
        % sum of squares of level averages of measured data
        ssq(1, factor) = sum(sum( (X_level_means{factor}).^2));
    end
    
    for i = 1 : n_interactions                                            
        factor_1 = interactions(i,1);
        factor_2 = interactions(i,2);
        % X_residuals = X - Xa - Xb
        X_residuals = X - X_level_means{factor_1} - X_level_means{factor_2};
        % Calculate cell means
       
        X_interaction_means{i} = zeros(size(X));
        for level_factor_1 = 1 : n_levels(factor_1)          % levels for first factor             
            for level_factor_2 = 1 : n_levels(factor_2)      % levels for second factor  
                tmp = zeros(size_data(1),1);
                found = find((F(:,factor_2) == level_factor_2) & ...  % find rows 
                             (F(:,factor_1) == level_factor_1));
                m = mean(X_residuals(found,:));                         % average over cell   
                tmp(found) = 1;
                X_interaction_means{i} = X_interaction_means{i} + tmp*m;
            end
        end
        ssq_interaction(1, i) = sum(sum( (X_interaction_means{i}).^2));
    end   
    
    
    % Interactions p-values are calculated through permutation of X - Xa - Xa
    % Do the permutations (do this, for now, for two factors)
    for j = 1 : n_perm
        
        % Permutations of experimental factors
        % Restrict permutations to the factor considered
        reverse_factors = n_factors : -1 : 1;
        for factor = 1 : n_factors
            other_factor       = reverse_factors(factor);
            X_permuted{factor} = zeros(size(X));
            for level = 1 : n_levels(other_factor)
                 found = find(F(:,other_factor) == level);
                 n_rows = length(found);                               % number of rows for level
                 perms  = randperm(n_rows);                            % random permutations of n_rows
                 permuted_level = found(perms);
                 X_permuted{factor}(found,:) = X(permuted_level,:);
            end
          
            % Level means for each factor 
            X_level_means{factor} = level_means(X_permuted{factor}, Fixed, factor);
   
            ssq(1 + j, factor) = sum(sum( (X_level_means{factor}).^2));
        end
        
        % Permutations for interactions. 
        for i = 1 : n_interactions               
            factor_1 = interactions(i,1);
            factor_2 = interactions(i,2);
            perms    = randperm(size(X,1));
            X_r      = X(perms, :);          % permuted data (permute whole data matrix)
        
            % Calculate level means experimental factors for permuted data 
            for factor = [factor_1 factor_2]
                X_level_means{factor} = level_means(X_r, Fixed, factor);
            end
            
            % X_residuals = Xr - Xa - Xb
            X_residuals = X_r - X_level_means{factor_1} - X_level_means{factor_2};
            X_interaction_means{i} = zeros(size(X));
            for level_factor_1 = 1 : n_levels(factor_1)                   % levels for first factor             
                for level_factor_2 = 1 : n_levels(factor_2)               % levels for second factor  
                    tmp = zeros(size_data(1),1);
                    found = find((F(:,factor_2) == level_factor_2) & ...  % find rows 
                                 (F(:,factor_1) == level_factor_1));
                    m = mean(X_residuals(found,:));                       % average over cell   
                    tmp(found) = 1;
                    X_interaction_means{i} = X_interaction_means{i} + tmp*m;
                end
            end
            ssq_interaction(1 + j, i) = sum(sum( (X_interaction_means{i}).^2));
        end           
        
    end        % permutations  
    
    % Calculate p-values
    % how many ssq's are larger than measurement ssq?
    for factor = 1 : n_factors
        p_factor(factor) = (size(find(ssq(2:n_perm + 1, factor) ...
                        >= ssq(1, factor)),1) + 1)/(n_perm);
    end
    for interaction = 1 : n_interactions
        p_interaction(interaction) = (size(find(ssq_interaction(2:n_perm + 1, interaction) ...
                                  >= ssq_interaction(1, interaction)),1) + 1)/(n_perm);
    end
     disp('p-values factors:')
     disp (p_factor)
     disp('p-values intractions')
     disp(p_interaction)
     p = [p_factor p_interaction];
 toc   
end            % function

function M = level_means(Y, Input, factor)
    % Calculate level means for factor in matrix Y
    size_Y = size(Y);
    M = zeros(size_Y);
    for level = 1 : Input.n_levels(factor)
        tmp = zeros(size_Y(1),1);
        found = find(Input.F(:,factor) == level);  % find rows that belong to level
        m = mean(Y(found,:));                      % calculate level mean 
        tmp(found) = 1;                            % flag the rows found
        M = M + tmp*m;
    end
end
 

    